#
# Copyright (c) 2010 Adam Tauno Williams <awilliam@whitemice.org>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#
from datetime           import datetime
from coils.foundation   import *
from coils.core         import *
from coils.core.logic   import UpdateCommand
from keymap             import COILS_NOTE_KEYMAP

class UpdateNote(UpdateCommand):
    __domain__    = "note"
    __operation__ = "set"

    def prepare(self, ctx, **params):
        self.keymap = COILS_NOTE_KEYMAP
        self.entity = Note
        UpdateCommand.prepare(self, ctx, **params)

    def parse_parameters(self, **params):
        UpdateCommand.parse_parameters(self, **params)
        if ('text' in params): self.values['text'] = params['text']
        self._context = params.get('context', None)

    def run(self):
        UpdateCommand.run(self)
        self.obj.modified = self._ctx.get_utctime()
        if (self._context is not None):
            if (isinstance(self._context, Appointment)):
                self.obj.appointment_id = self._context.object_id
            elif (isinstance(self._context, Project)):
                self.obj.project_id = self._context.object_id
            elif (isinstance(self._context, Contact)):
                self.obj.company_id = self._context.object_id
            elif (isinstance(self._context, Enterprise)):
                self.obj.company_id = self._context.object_id
        handle = BLOBManager.Create(self.obj.get_path())
        handle.write(self.obj.content)
        BLOBManager.Close(handle)
